# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.conf.urls import url, include

urlpatterns = [
    # Frontend
    url(r'^wizard/', include('testapp.urls', namespace='testapp'))
]



"""
This is a URLconf to be loaded by tests.py. Add any URLs needed for tests only.
"""

from django.conf.urls import url

from .forms import ContactWizardAPIView, NamedContactWizardAPIView, SubStepContactWizardAPIView, \
    NamedSubStepContactWizardAPIView, ComplexNamedSubStepContactWizardAPIView

test_wizard1 = ContactWizardAPIView.as_view(url_name='wizard_step')
test_wizard2 = NamedContactWizardAPIView.as_view(url_name='wizard_step')
test_wizard3 = SubStepContactWizardAPIView.as_view(url_name='wizard_step')
test_wizard4 = NamedSubStepContactWizardAPIView.as_view(url_name='wizard_step')
test_wizard5 = ComplexNamedSubStepContactWizardAPIView.as_view(url_name='wizard_step')


urlpatterns = [
    # Steps will be index ints
    url(r'^wizard/(?P<step>.+)/$', test_wizard1, name='wizard_step'),
    url(r'^wizard/$', test_wizard1, name='wizard'),

    # Steps will be (hopefully descriptive) strings
    url(r'^named-wizard/(?P<step>.+)/$', test_wizard2, name='named_wizard_step'),
    url(r'^named-wizard/$', test_wizard2, name='named_wizard'),

    # Steps will be index ints
    url(r'^substep-wizard/(?P<step>.+)/$', test_wizard3, name='substep_wizard_step'),
    url(r'^substep-wizard/$', test_wizard3, name='substep_wizard'),

    # Steps will be (hopefully descriptive) strings
    url(r'^named-substep-wizard/(?P<step>.+)/$', test_wizard4, name='named_substep_wizard_step'),
    url(r'^named-substep-wizard/$', test_wizard4, name='named_substep_wizard'),

    # Steps will be (hopefully descriptive) strings
    url(r'^complex-named-substep-wizard/(?P<step>.+)/(?P<substep>.+)/$', test_wizard5, name='complex_named_substep_wizard_step'),
    url(r'^complex-named-substep-wizard/(?P<step>.+)/$', test_wizard5, name='complex_named_substep_wizard_step'),
    url(r'^complex-named-substep-wizard/$', test_wizard5, name='complex_named_substep_wizard'),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json

from django.core.urlresolvers import reverse
from django.http.response import JsonResponse
from django.test.testcases import TestCase
from django.test.utils import override_settings

from formtools_addons.enums import HTTP_APPLICATION_JSON
from formtools_addons.wizard.views.wizardapi import WizardAPIView


@override_settings(
    ROOT_URLCONF='tests.wizard.wizardapitests.urls',
)
class WizardAPITests(TestCase):
    DEFAULT_HEADERS = {'HTTP_ACCEPT': HTTP_APPLICATION_JSON}

    ####################################################################################################################
    # Regular wizard, index based
    ####################################################################################################################
    def test_get_wizard_without_step(self):
        response = self.client.get(reverse('wizard'), **self.DEFAULT_HEADERS)
        assert response.status_code == 200
        assert self._get_response_data(response)['current_step'] == '0'

    def test_get_wizard_data_step(self):
        response = self.client.get(reverse('wizard_step', kwargs={'step': 'data'}), **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        data = self._get_response_data(response)
        assert data['structure'] == ['0', '1']

    def test_get_wizard_with_step(self):
        response = self.client.get(reverse('wizard_step', kwargs={'step': '1'}), **self.DEFAULT_HEADERS)
        assert response.status_code == 200
        assert self._get_response_data(response)['current_step'] == '1'

    def test_post_wizard_step_incomplete(self):
        data = {
            'name': 'test'
        }
        response = self.client.post(reverse('wizard_step', kwargs={'step': '0'}), data, **self.DEFAULT_HEADERS)
        assert response.status_code == 400

    def test_post_wizard_step(self):
        # Perform step 0
        input_data1 = {
            'name': 'test',
            'thirsty': True
        }
        response = self.client.post(reverse('wizard_step', kwargs={'step': '0'}), input_data1, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == '1'  # Next step's name
        assert data['done'] is False
        assert data['steps']['0']['data']['name'] == input_data1['name']
        assert data['steps']['0']['data']['thirsty'] == input_data1['thirsty']
        assert data['steps']['0']['valid'] is True
        assert data['steps']['1']['valid'] is False

        # Perform step 1
        input_data2 = {
            'address1': 'Address 1',
            'address2': 'Address 2'
        }
        response = self.client.post(reverse('wizard_step', kwargs={'step': '1'}), input_data2, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # Next step's name
        assert data['done'] is True
        assert data['steps']['1']['data']['address1'] == input_data2['address1']
        assert data['steps']['1']['data']['address2'] == input_data2['address2']
        assert data['steps']['1']['valid'] is True

        # Fetch data
        response = self.client.get(reverse('wizard_step', kwargs={'step': 'data'}), {}, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # There should not be another step
        assert data['done'] is True
        assert data['steps']['0']['data']['name'] == input_data1['name']
        assert data['steps']['0']['data']['thirsty'] == input_data1['thirsty']
        assert data['steps']['0']['valid'] is True
        assert data['steps']['1']['data']['address1'] == input_data2['address1']
        assert data['steps']['1']['data']['address2'] == input_data2['address2']
        assert data['steps']['1']['valid'] is True

        # Commit data
        response = self.client.post(reverse('wizard_step', kwargs={'step': 'commit'}), {}, **self.DEFAULT_HEADERS)

        assert response.status_code == 302

    ####################################################################################################################
    # Regular wizard, name based
    ####################################################################################################################
    def test_get_named_wizard_without_step(self):
        response = self.client.get(reverse('named_wizard'), **self.DEFAULT_HEADERS)
        assert response.status_code == 200
        assert self._get_response_data(response)['current_step'] == 'page1'

    def test_get_named_wizard_data_step(self):
        response = self.client.get(reverse('named_wizard_step', kwargs={'step': 'data'}), **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        data = self._get_response_data(response)
        assert data['structure'] == ['page1', 'page2']

    def test_get_named_wizard_with_step(self):
        response = self.client.get(reverse('named_wizard_step', kwargs={'step': 'page2'}), **self.DEFAULT_HEADERS)
        assert response.status_code == 200
        assert self._get_response_data(response)['current_step'] == 'page2'

    def test_post_named_wizard_step_incomplete(self):
        data = {
            'name': 'test'
        }
        response = self.client.post(reverse('named_wizard_step', kwargs={'step': 'page1'}), data,
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 400

    def test_post_named_wizard_step(self):
        # Perform step 0
        input_data1 = {
            'name': 'test',
            'thirsty': True
        }
        response = self.client.post(reverse('named_wizard_step', kwargs={'step': 'page1'}), input_data1,
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == 'page2'  # Next step's name
        assert data['done'] is False
        assert data['steps']['page1']['data']['name'] == input_data1['name']
        assert data['steps']['page1']['data']['thirsty'] == input_data1['thirsty']
        assert data['steps']['page1']['valid'] is True
        assert data['steps']['page2']['valid'] is False

        # Perform step 1
        input_data2 = {
            'address1': 'Address 1',
            'address2': 'Address 2'
        }
        response = self.client.post(reverse('named_wizard_step', kwargs={'step': 'page2'}), input_data2,
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # Next step's name
        assert data['done'] is True
        assert data['steps']['page2']['data']['address1'] == input_data2['address1']
        assert data['steps']['page2']['data']['address2'] == input_data2['address2']
        assert data['steps']['page2']['valid'] is True

        # Fetch data
        response = self.client.get(reverse('named_wizard_step', kwargs={'step': 'data'}), {}, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # There should not be another step
        assert data['done'] is True
        assert data['steps']['page1']['data']['name'] == input_data1['name']
        assert data['steps']['page1']['data']['thirsty'] == input_data1['thirsty']
        assert data['steps']['page1']['valid'] is True
        assert data['steps']['page2']['data']['address1'] == input_data2['address1']
        assert data['steps']['page2']['data']['address2'] == input_data2['address2']
        assert data['steps']['page2']['valid'] is True

        # Commit data
        response = self.client.post(reverse('named_wizard_step', kwargs={'step': 'commit'}), {}, **self.DEFAULT_HEADERS)

        assert response.status_code == 302

    ####################################################################################################################
    # Substep wizard, index based
    ####################################################################################################################
    def test_get_substep_wizard_without_step(self):
        response = self.client.get(reverse('substep_wizard'), **self.DEFAULT_HEADERS)
        assert response.status_code == 200
        assert self._get_response_data(response)['current_step'] == '0|step1.1'

    def test_get_substep_wizard_data_step(self):
        response = self.client.get(reverse('substep_wizard_step', kwargs={'step': 'data'}), **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        data = self._get_response_data(response)
        assert data['structure'] == ['0|step1.1', '0|step1.2', '1|step2.1']

    def test_get_substep_wizard_with_step(self):
        response = self.client.get(reverse('substep_wizard_step', kwargs={'step': '0|step1.2'}), **self.DEFAULT_HEADERS)
        assert response.status_code == 200
        assert self._get_response_data(response)['current_step'] == '0|step1.2'

    def test_post_substep_wizard_step_incomplete(self):
        data = {
            'name': 'test'
        }
        response = self.client.post(reverse('substep_wizard_step', kwargs={'step': '0|step1.1'}), data,
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 400

    def test_post_substep_wizard_step(self):
        # Perform step 0
        input_data1 = {
            'name': 'test',
            'thirsty': True
        }
        response = self.client.post(reverse('substep_wizard_step', kwargs={'step': '0|step1.1'}), input_data1,
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == '0|step1.2'  # Next step's name
        assert data['done'] is False
        assert data['steps']['0|step1.1']['data']['name'] == input_data1['name']
        assert data['steps']['0|step1.1']['data']['thirsty'] == input_data1['thirsty']
        assert data['steps']['0|step1.1']['valid'] is True
        assert data['steps']['0|step1.2']['valid'] is False
        assert data['steps']['1|step2.1']['valid'] is False

        # Perform step 1
        input_data2 = {
            'address1': 'Address 1',
            'address2': 'Address 2'
        }
        response = self.client.post(reverse('substep_wizard_step', kwargs={'step': '0|step1.2'}), input_data2,
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == '1|step2.1'  # Next step's name
        assert data['done'] is False
        assert data['steps']['0|step1.2']['data']['address1'] == input_data2['address1']
        assert data['steps']['0|step1.2']['data']['address2'] == input_data2['address2']
        assert data['steps']['0|step1.2']['valid'] is True
        assert data['steps']['1|step2.1']['valid'] is False

        # Perform step 2
        input_data3 = {
            'random_crap': 'Blablabla'
        }
        response = self.client.post(reverse('substep_wizard_step', kwargs={'step': '1|step2.1'}), input_data3, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # Next step's name
        assert data['done'] is True
        assert data['steps']['1|step2.1']['data']['random_crap'] == input_data3['random_crap']
        assert data['steps']['1|step2.1']['valid'] is True

        # Fetch data
        response = self.client.get(reverse('substep_wizard_step', kwargs={'step': 'data'}), {}, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # There should not be another step
        assert data['done'] is True
        assert data['steps']['0|step1.1']['data']['name'] == input_data1['name']
        assert data['steps']['0|step1.1']['data']['thirsty'] == input_data1['thirsty']
        assert data['steps']['0|step1.1']['valid'] is True
        assert data['steps']['0|step1.2']['data']['address1'] == input_data2['address1']
        assert data['steps']['0|step1.2']['data']['address2'] == input_data2['address2']
        assert data['steps']['0|step1.2']['valid'] is True
        assert data['steps']['1|step2.1']['data']['random_crap'] == input_data3['random_crap']
        assert data['steps']['1|step2.1']['valid'] is True

        # Commit data
        response = self.client.post(reverse('substep_wizard_step', kwargs={'step': 'commit'}), {},
                                    **self.DEFAULT_HEADERS)

        assert response.status_code == 302

    ####################################################################################################################
    # Substep wizard, name based
    ####################################################################################################################
    def test_get_named_substep_wizard_without_step(self):
        response = self.client.get(reverse('named_substep_wizard'), **self.DEFAULT_HEADERS)
        assert response.status_code == 200
        assert self._get_response_data(response)['current_step'] == 'page1|step1.1'

    def test_get_named_substep_wizard_data_step(self):
        response = self.client.get(reverse('named_substep_wizard_step', kwargs={'step': 'data'}),
                                   **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        data = self._get_response_data(response)
        assert data['structure'] == ['page1|step1.1', 'page1|step1.2', 'page2|step2.1']

    def test_get_named_substep_wizard_with_step(self):
        response = self.client.get(reverse('named_substep_wizard_step', kwargs={'step': 'page1|step1.2'}),
                                   **self.DEFAULT_HEADERS)
        assert response.status_code == 200
        assert self._get_response_data(response)['current_step'] == 'page1|step1.2'

    def test_post_named_substep_wizard_step_incomplete(self):
        data = {
            'name': 'test'
        }
        response = self.client.post(reverse('named_substep_wizard_step', kwargs={'step': 'page1|step1.1'}), data,
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 400

    def test_post_named_substep_wizard_step(self):
        # Perform step 0
        input_data1 = {
            'name': 'test',
            'thirsty': True
        }
        response = self.client.post(reverse('named_substep_wizard_step', kwargs={'step': 'page1|step1.1'}), input_data1,
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == 'page1|step1.2'  # Next step's name
        assert data['done'] is False
        assert data['steps']['page1|step1.1']['data']['name'] == input_data1['name']
        assert data['steps']['page1|step1.1']['data']['thirsty'] == input_data1['thirsty']
        assert data['steps']['page1|step1.1']['valid'] is True
        assert data['steps']['page1|step1.2']['valid'] is False
        assert data['steps']['page2|step2.1']['valid'] is False

        # Perform step 1
        input_data2 = {
            'address1': 'Address 1',
            'address2': 'Address 2'
        }
        response = self.client.post(reverse('named_substep_wizard_step', kwargs={'step': 'page1|step1.2'}), input_data2,
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == 'page2|step2.1'  # Next step's name
        assert data['done'] is False
        assert data['steps']['page1|step1.2']['data']['address1'] == input_data2['address1']
        assert data['steps']['page1|step1.2']['data']['address2'] == input_data2['address2']
        assert data['steps']['page1|step1.2']['valid'] is True
        assert data['steps']['page2|step2.1']['valid'] is False

        # Perform step 2
        input_data3 = {
            'random_crap': 'Blablabla'
        }
        response = self.client.post(reverse('named_substep_wizard_step', kwargs={'step': 'page2|step2.1'}), input_data3,
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # Next step's name
        assert data['done'] is True
        assert data['steps']['page2|step2.1']['data']['random_crap'] == input_data3['random_crap']
        assert data['steps']['page2|step2.1']['valid'] is True

        # Fetch data
        response = self.client.get(reverse('named_substep_wizard_step', kwargs={'step': 'data'}), {},
                                   **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # There should not be another step
        assert data['done'] is True
        assert data['steps']['page1|step1.1']['data']['name'] == input_data1['name']
        assert data['steps']['page1|step1.1']['data']['thirsty'] == input_data1['thirsty']
        assert data['steps']['page1|step1.1']['valid'] is True
        assert data['steps']['page1|step1.2']['data']['address1'] == input_data2['address1']
        assert data['steps']['page1|step1.2']['data']['address2'] == input_data2['address2']
        assert data['steps']['page1|step1.2']['valid'] is True
        assert data['steps']['page2|step2.1']['data']['random_crap'] == input_data3['random_crap']
        assert data['steps']['page2|step2.1']['valid'] is True

        # Commit data
        response = self.client.post(reverse('named_substep_wizard_step', kwargs={'step': 'commit'}), {},
                                    **self.DEFAULT_HEADERS)

        assert response.status_code == 302

    def test_post_complex_named_substep_wizard_step(self):
        # Perform step 0
        input_data1 = {
            'name': 'test',
            'thirsty': True
        }
        response = self.client.post(reverse('complex_named_substep_wizard_step', kwargs={'step': 'page1|step1.1'}),
                                    input_data1, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == 'page1|step1.2'  # Next step's name
        assert data['done'] is False

        # Perform step 1
        input_data2 = {
            'address1': 'Address 1',
            'address2': 'Address 2'
        }
        response = self.client.post(reverse('complex_named_substep_wizard_step', kwargs={'step': 'page1|step1.2'}),
                                    input_data2, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == 'page1|step1.3'  # Next step's name
        assert data['done'] is False

        # Perform step 2
        input_data3 = {
            'random_crap': 'Blablabla'
        }
        response = self.client.post(reverse('complex_named_substep_wizard_step', kwargs={'step': 'page1|step1.3'}),
                                    input_data3, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == 'page2|step2.1'  # Next step's name
        assert data['done'] is False

        # Perform step 3
        input_data4 = {
            'name': 'test2',
            'thirsty': True
        }
        response = self.client.post(reverse('complex_named_substep_wizard_step', kwargs={'step': 'page2|step2.1'}),
                                    input_data4, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == 'page2|step2.2'  # Next step's name
        assert data['done'] is False

        ############################################################################################################
        # NOW WE JUMP BACK TO STEP 1.1!
        ############################################################################################################
        response = self.client.post(reverse('complex_named_substep_wizard_step', kwargs={'step': 'goto/page1|step1.1'}),
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == 'page1|step1.1'  # Next step's name
        assert data['done'] is False
        assert data['valid'] is False
        assert 'page2|step2.2' in data['steps']

        # We change the 'name' value to 'hurray' (see tests/wizard/wizardapitests/url.py)
        input_data5 = {
            'name': 'hurray',
            'thirsty': True
        }
        response = self.client.post(reverse('complex_named_substep_wizard_step', kwargs={'step': 'page1|step1.1'}),
                                    input_data5, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == 'page1|step1.2'  # Next step's name
        assert data['done'] is False
        assert data['valid'] is True
        assert 'page2|step2.2' not in data['steps']

        ############################################################################################################
        # NOW WE JUMP FORWARD TO STEP 2.1!
        ############################################################################################################
        response = self.client.post(reverse('complex_named_substep_wizard_step', kwargs={'step': 'goto/page2|step2.1'}),
                                    **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] == 'page2|step2.1'  # Next step's name
        assert data['done'] is False
        assert data['valid'] is True

        # Perform step 3
        input_data6 = {
            'name': 'test2',
            'thirsty': True
        }
        response = self.client.post(reverse('complex_named_substep_wizard_step', kwargs={'step': 'page2|step2.1'}),
                                    input_data6, **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # Next step's name
        assert data['done'] is True
        assert data['valid'] is True

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # Next step's name
        assert data['done'] is True

        # Fetch data
        response = self.client.get(reverse('complex_named_substep_wizard_step', kwargs={'step': 'data'}), {},
                                   **self.DEFAULT_HEADERS)
        assert response.status_code == 200

        # Get data
        data = self._get_response_data(response)
        assert data['current_step'] is None  # There should not be another step
        assert data['done'] is True
        assert data['steps']['page1|step1.1']['data']['name'] == input_data5['name']
        assert data['steps']['page1|step1.1']['data']['thirsty'] == input_data5['thirsty']
        assert data['steps']['page1|step1.1']['valid'] is True
        assert data['steps']['page1|step1.2']['data']['address1'] == input_data2['address1']
        assert data['steps']['page1|step1.2']['data']['address2'] == input_data2['address2']
        assert data['steps']['page1|step1.2']['valid'] is True
        assert data['steps']['page1|step1.3']['data']['random_crap'] == input_data3['random_crap']
        assert data['steps']['page1|step1.3']['valid'] is True
        assert data['steps']['page2|step2.1']['data']['name'] == input_data4['name']
        assert data['steps']['page2|step2.1']['data']['thirsty'] == input_data4['thirsty']
        assert data['steps']['page2|step2.1']['valid'] is True
        assert 'page2|step2.2' not in data['steps']

        # Commit data
        response = self.client.post(reverse('complex_named_substep_wizard_step', kwargs={'step': 'commit'}), {},
                                    **self.DEFAULT_HEADERS)

        assert response.status_code == 302

    def _get_response_data(self, response):
        assert isinstance(response, JsonResponse)
        return json.loads(response.content.decode('utf-8'))




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import url
from django.views.decorators.csrf import ensure_csrf_cookie

from .views import TestWizard, TestWizardContainer


test_wizard_container = TestWizardContainer.as_view()
test_wizard = TestWizard.as_view(url_name='testapp:accordeon_wizard_step')

urlpatterns = [
    url(r'^(?P<step>.+)/(?P<substep>.+)/$', ensure_csrf_cookie(test_wizard), name='accordeon_wizard_step'),
    url(r'^(?P<step>.+)/$', ensure_csrf_cookie(test_wizard), name='accordeon_wizard_step'),
    url(r'^$', test_wizard_container, name='accordeon_wizard'),
]



import os
import tempfile

from django import forms
from django.core.files.storage import FileSystemStorage
from django.forms.formsets import formset_factory
from django.shortcuts import redirect

from formtools_addons.wizard.views.wizardapi import WizardAPIView

temp_storage_location = tempfile.mkdtemp(dir=os.environ.get('DJANGO_TEST_TEMP_DIR'))
temp_storage = FileSystemStorage(location=temp_storage_location)


class Page1(forms.Form):
    name = forms.CharField(max_length=100)
    thirsty = forms.BooleanField()


class Page2(forms.Form):
    address1 = forms.CharField(max_length=100)
    address2 = forms.CharField(max_length=100)
    # file1 = forms.FileField()


class Page3(forms.Form):
    random_crap = forms.CharField(max_length=100)

Page4 = formset_factory(Page3, extra=2)


class ContactWizardAPIView(WizardAPIView):
    storage_name = 'formtools.wizard.storage.session.SessionStorage'
    form_list = (Page1, Page2)

    def done(self, form_list, **kwargs):
        return redirect('/next-page/')


class NamedContactWizardAPIView(WizardAPIView):
    storage_name = 'formtools.wizard.storage.session.SessionStorage'
    form_list = (
        ('page1', Page1),
        ('page2', Page2)
    )

    def done(self, form_list, **kwargs):
        return redirect('/next-page/')


class SubStepContactWizardAPIView(WizardAPIView):
    storage_name = 'formtools.wizard.storage.session.SessionStorage'
    form_list = (
        (
            ('step1.1', Page1),
            ('step1.2', Page2)
        ),
        (
            ('step2.1', Page3),
        )
    )

    def done(self, form_list, **kwargs):
        return redirect('/next-page/')


class NamedSubStepContactWizardAPIView(WizardAPIView):
    storage_name = 'formtools.wizard.storage.session.SessionStorage'
    form_list = (
        ('page1', (
            ('step1.1', Page1),
            ('step1.2', Page2)
        )),
        ('page2', (
            ('step2.1', Page3),
        ))
    )

    def done(self, form_list, **kwargs):
        return redirect('/next-page/')


def show_page2_step2(wizard):
    data = wizard.get_cleaned_data_for_step('page1|step1.1') or {}
    return data.get('name', '') != 'hurray'


class ComplexNamedSubStepContactWizardAPIView(WizardAPIView):
    storage_name = 'formtools.wizard.storage.session.SessionStorage'
    form_list = (
        ('page1', (
            ('step1.1', Page1),
            ('step1.2', Page2),
            ('step1.3', Page3)
        )),
        ('page2', (
            ('step2.1', Page1),
            ('step2.2', Page2),
        ))
    )

    condition_dict = {
        'page2|step2.2': show_page2_step2
    }

    def done(self, form_list, **kwargs):
        return redirect('/next-page/')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.http.response import JsonResponse
from django.template.context import Context
from django.template.loader import get_template
from django.views.generic.base import TemplateView

from formtools_addons.wizard.views.wizardapi import WizardAPIView
from .forms import *


class TestWizardContainer(TemplateView):
    template_name = 'formtools_addons/accordeon_wizard_app.html'


def show_testform_2_conditional(wizard):
    cleaned_data = wizard.get_cleaned_data_for_step('page1|testform1') or {}
    return cleaned_data.get('sender', None) != 'dirk@gmail.com'


def show_testform_5_conditional(wizard):
    cleaned_data = wizard.get_cleaned_data_for_step('page1|testform2') or {}
    return cleaned_data.get('message', '') != 'test'


class TestWizard(WizardAPIView):
    storage_name = 'formtools.wizard.storage.session.SessionStorage'

    form_list = [
        ('page1', (
            ('testform1', TestForm1),
            ('testform2', TestForm2),
            ('testform3', TestForm3),
        )),
        ('page2', (
            ('testform4', TestForm4),
            ('testform5', TestForm5),
        )),
    ]

    condition_dict = {
        'page1|testform2': show_testform_2_conditional,
        'page2|testform5': show_testform_5_conditional
    }

    templates = {
        'page1|testform1': 'testapp/testform1.html',
        'page1|testform2': 'testapp/testform2.html',
    }

    def render_form(self, step, form):
        template = get_template(self.templates.get(step, 'testapp/default_form.html'))

        context = Context()
        context['form'] = form

        return template.render(context)

    def done(self, form_list, **kwargs):
        """
        Wizard finish handler.
          The idea is to persist your data here
        """
        return JsonResponse({'next_url': '/next-page/'})



# -*- coding: utf-8 -*-
import os
import environ

root = environ.Path(__file__) - 2

env = environ.Env(
    DEBUG=(bool, True),
)

DEBUG = env('DEBUG')

# noinspection PyCallingNonCallable
SITE_ROOT = root()

DATABASES = {
    'default': env.db(default='sqlite:///%s/dev.sqlite.db' % SITE_ROOT),
}

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.staticfiles',

    'formtools',
    'formtools_addons',

    'testapp',
]

ROOT_URLCONF = 'accordeon_wizard.urls'

SECRET_KEY = 'spam-spam-spam-spam'

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'spam-and-eggs'
    }
}

SESSION_ENGINE = 'django.contrib.sessions.backends.cache'

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',

    'formtools_addons.middleware.JSONMiddleware'
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(SITE_ROOT, 'accordeon_wizard/templates')
        ]
        ,
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

SITE_ID = 1

MEDIA_ROOT = 'media'
STATIC_ROOT = 'static'

STATIC_URL = '/static/'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'formtools_addons.wizard.wizardapi': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
    },
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms


class BaseForm(forms.Form):
    def is_valid(self):
        return super(BaseForm, self).is_valid()


class TestForm1(BaseForm):
    subject = forms.CharField(max_length=100)
    sender = forms.EmailField()


class TestForm2(BaseForm):
    message = forms.CharField(widget=forms.Textarea)
    multivalue = forms.MultipleChoiceField(choices=[('testa', 'Test A'), ('testb', 'Test B')])


class TestForm3(BaseForm):
    field1 = forms.CharField(max_length=100)
    field2 = forms.CharField(max_length=100)


class TestForm4(BaseForm):
    field1 = forms.IntegerField()


class TestForm5(BaseForm):
    name = forms.ChoiceField(choices=[('test1', 'Test 1'), ('test2', 'Test 2')])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys

import setuptools

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

__version__ = '0.2.1'

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setuptools.setup(
    name='django2-formtools-addons',
    version=__version__,
    description="""'Addons for Django 2 Formtools'""",
    long_description=readme + '\n\n' + history,
    author='VikingCo',
    author_email='technology@vikingco.com',
    url='https://github.com/hpe95/django-formtools-addons',
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=["django >= 2.1", "six >= 1.9"],
    extras_require={},
    license="BSD",
    zip_safe=False,
    keywords='django2-formtools-addons',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
